/* File: mkwsd.c 
 
* Copyright (c) 2002 James A. Strother 
* 
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details. 
*  
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
* 
*/ 
 
/* Includes */ 
#include <stdio.h> 
#include <stdlib.h> 
#include <math.h> 
#include <string.h> 
#include <fftw3.h> 
#include "osiv.h" 
 
void mkwsd_error(const char* m) 
{ 
  fprintf(stderr,"mkwsd error: %s\n", m); 
  exit(EXIT_FAILURE); 
} 
 
void mkwsd_puint(char* arg, 
		 long* number) 
{ 
  char* t; 
  *number = strtol(arg,&t,10); 
  if (*t != '\0') 
    mkwsd_error("unable to parse number"); 
} 
 
void mkwsd_pustr(char* arg, 
		 char* str) 
{ 
  strcpy(str,arg); 
} 
 
void mkwsd_parse(int argc, 
		 char** argv, 
		 char* fno, 
		 long* row, 
		 long* col) 
{ 
  int i; 
 
  /* Go to first real argument */ 
  argc--; 
  argv++; 
 
  /* Set null argument states */ 
  *fno = '\0'; 
  *row = -1; 
  *col = -1; 
 
  /* Traverse arguments */ 
  i = 0; 
  while (i < argc) 
    { 
      char* c = *(argv+i++); 
       
      if ((*c != '-') || (strlen(c)>2)) 
	mkwsd_error("unable to parse command"); 
 
      switch (*(c+1)) 
	{ 
	case 'o': 
	  if (i >= argc) 
	    mkwsd_error("can't parse arguments"); 
	  mkwsd_pustr(*(argv+i++),fno); 
	  break; 
	case 'r': 
	  if (i >= argc) 
	    mkwsd_error("can't parse arguments"); 
	  mkwsd_puint(*(argv+i++),row); 
	  break; 
	case 'c': 
	  if (i >= argc) 
	    mkwsd_error("can't parse arguments"); 
	  mkwsd_puint(*(argv+i++),col); 
	  break; 
	default: 
	  mkwsd_error("unrecognized option"); 
	  break; 
	} 
    } 
 
  /* Verify argments */ 
  if (*fno == '\0') 
    mkwsd_error("missing output file argument"); 
  if (*row == -1) 
    mkwsd_error("missing row argument"); 
  if (*col == -1) 
    mkwsd_error("missing col argument"); 
 
} 
 
void mkwsd(FILE* fo, 
	   long rows, 
	   long cols) 
{ 
  fftw_real *ft; 
  fftw_plan planf,planb; 
 
  /* Make array */ 
  ft = (fftw_real*)fftw_malloc(2*rows*(cols/2+1)*sizeof(fftw_real)); 
  if (!ft) mkwsd_error("unable to allocate memory"); 
 
  /* Construct plans */ 
  planf = fftw_plan_dft_r2c_2d(rows,cols,ft,(fftw_complex*)ft,FFTW_PATIENT); 
  planb = fftw_plan_dft_c2r_2d(rows,cols,(fftw_complex*)ft,ft,FFTW_PATIENT); 
 
  /* Write wisdom */ 
  fftw_export_wisdom_to_file(fo); 
   
  /* Clean up */ 
  fftw_free(ft); 
  fftw_destroy_plan(planf); 
  fftw_destroy_plan(planb); 
 
} 
 
int main(int argc, char** argv) 
{ 
  FILE *fo; 
  char *fno; 
  long rows,cols; 
 
  /* Allocate memory for strings */ 
  fno = (char*)malloc(OSIV_STR_LEN+1); 
  if (!fno)  
    mkwsd_error("unable to allocate memory"); 
 
  /* Parse arguments */ 
  mkwsd_parse(argc,argv,fno,&rows,&cols); 
 
  /* Open output file */ 
  if (!strcmp(fno,"-")) 
    fo = stdout; 
  else 
    fo = fopen(fno,"w"); 
 
  if (!fo) 
    mkwsd_error("unable to open output file"); 
 
  /* Print arguments */ 
  mkwsd(fo,rows,cols); 
 
  /* Close file */ 
  fclose(fo); 
 
  /* Clean up */ 
  free(fno); 
 
  /* Return */ 
  exit(EXIT_SUCCESS); 
} 
