/* File: load.c

* Copyright (c) 2002 James A. Strother
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/

#include "osiv.h"

osiv_file* OsivOpenIO (FILE* fp)
{
  osiv_file* file;
  osiv_uint16 val;

  if (!fp) return NULL;

  file = (osiv_file*)malloc(sizeof(osiv_file));
  if (!file) return NULL;

  file->stream = fp;
  if (!fp || fgetc(fp)!=0xff || fgetc(fp)!=0x00)
    return NULL;

  if (fread(&val,sizeof(osiv_uint16),1,fp) != 1)
    return NULL;

  if ((file->swap_bytes = (val & 0xff00)) != 0)
    file->swap_bits = (val & 0x8000);
  else
    file->swap_bits = (val & 0x0080);

  if ((file->iotag_size = fgetc(fp)) == EOF ||
      (file->ioint_size = fgetc(fp)) == EOF ||
      (file->ioreal_size = fgetc(fp)) == EOF)
    return NULL;

  if (file->iotag_size != sizeof(osiv_iotag) ||
      file->ioint_size != sizeof(osiv_ioint) ||
      file->ioreal_size != sizeof(osiv_ioreal))
    return NULL;

  if ((file->ver_first = fgetc(fp)) == EOF ||
      (file->ver_second = fgetc(fp)) == EOF ||
      (file->ver_third = fgetc(fp)) == EOF)
    return NULL;

  if ((file->pre_len = ftell(fp)) == -1)
    return NULL;

  return file;
}

osiv_file* OsivOpen (const char* fn)
{
  return OsivOpenIO(fopen(fn,"rb"));
}

osiv_bool OsivClose (osiv_file* file)
{
  if (fclose(file->stream) == EOF)
    return OSIV_TRUE;

  free(file);
  return OSIV_FALSE;
}

osiv_bool OsivRead (void* ptr,
		    size_t size,
		    size_t nitems,
		    osiv_file* file)
{

  if (fread(ptr,size,nitems,file->stream) != nitems)
    return OSIV_TRUE;

  if (file->swap_bytes)
    {
      switch (size)
	{
	case 1:
	  break;
	case 2:
	  {
	    register size_t count = nitems;
	    register osiv_uint16 val0,val1;
	    register osiv_uint16 *bptr = (osiv_uint16*)ptr;

	    while (count--)
	      {
		val0 = *bptr;
		val1 =  (val0 & 0x00ff)<<8;
		val1 |= (val0 & 0xff00)>>8;
		*bptr++ = val1;
	      }
	  }
	  break;
	case 4:
	  {
	    register size_t count = nitems;
	    register osiv_uint32 val0,val1;
	    register osiv_uint32 *bptr = (osiv_uint32*)ptr;

	    while (count--)
	      {
		val0 = *bptr;
		val1 =  (val0 & 0x000000ff)<<24;
		val1 |= (val0 & 0x0000ff00)<<8;
		val1 |= (val0 & 0x00ff0000)>>8;
		val1 |= (val0 & 0xff000000)>>24;
		*bptr++ = val1;
	      }
	  }
	  break;
	case 8:
	  {
	    register size_t count = nitems;
	    register osiv_uint64 val0,val1;
	    register osiv_uint64 *bptr = (osiv_uint64*)ptr;

	    while (count--)
	      {
		val0 = *bptr;
		val1 =  (val0 & 0x00000000000000ff)<<56;
		val1 |= (val0 & 0x000000000000ff00)<<40;
		val1 |= (val0 & 0x0000000000ff0000)<<24;
		val1 |= (val0 & 0x00000000ff000000)<<8;
		val1 |= (val0 & 0x000000ff00000000)>>8;
		val1 |= (val0 & 0x0000ff0000000000)>>24;
		val1 |= (val0 & 0x00ff000000000000)>>40;
		val1 |= (val0 & 0xff00000000000000)>>56;
		*bptr++ = val1;
	      }
	  }
	  break;
	default:
	  {
	    register char* bptr = (char*)ptr;
	    register size_t count = nitems;

	    while (count--)
	      {
		register size_t i;
		for (i = 0; i < size/2; i++)
		  {
		    register char val0;
		    val0 = *(bptr+i);
		    *(bptr+i) = *(bptr+size-i-1);
		    *(bptr+size-i-1) = val0;
		  }
		bptr += size;
	      }
	  }
	  break;
	}
    }

  if (file->swap_bits)
    {
      register char c0,c1;
      register char* bptr = (char*)ptr;
      register size_t count = size*nitems;

      while (count--)
	{
	  c0 = *bptr;
	  c1 =  (c0 & 0x01)<<7;
	  c1 |= (c0 & 0x02)<<5;
	  c1 |= (c0 & 0x04)<<3;
	  c1 |= (c0 & 0x08)<<1;
	  c1 |= (c0 & 0x10)>>7;
	  c1 |= (c0 & 0x20)>>5;
	  c1 |= (c0 & 0x40)>>3;
	  c1 |= (c0 & 0x80)>>1;
	  *bptr++ = c1;
	}
    }

  return OSIV_FALSE;
}

osiv_bool OsivRewind (osiv_file* fp)
{
  return (fseek(fp->stream,fp->pre_len,SEEK_SET) == -1);
}

osiv_bool OsivTell (osiv_file* fp,
		    long* pos)
{
  return ((*pos = ftell(fp->stream)) == -1);
}

osiv_bool OsivSeek (osiv_file* fp,
		    long offset,
		    int whence)
{
  return (fseek(fp->stream,offset,whence) == -1);
}

osiv_bool OsivReadVer (osiv_file* fp,
		       char* ver)
{
  return (sprintf(ver,"%d",fp->ver_first) < 0 ||
	  sprintf(ver+strlen(ver),".%d",fp->ver_second) < 0 ||
	  sprintf(ver+strlen(ver),".%d",fp->ver_third) < 0);
}

osiv_bool OsivReadTag (osiv_file* file,
		       osiv_iotag* tag)
{
  return (OsivRead(tag,sizeof(osiv_iotag),1,file));
}

osiv_bool OsivReadArgInt (osiv_file* file,
			  osiv_ioint* key,
			  osiv_ioint* val)
{
  osiv_ioint blen;
  return (OsivRead(&blen,sizeof(osiv_ioint),1,file) ||
	  OsivRead(key,sizeof(osiv_ioint),1,file) ||
	  OsivRead(val,sizeof(osiv_ioint),1,file) ||
	  blen != 2*sizeof(osiv_ioint));
}

osiv_bool OsivReadArgStr (osiv_file* file,
			  osiv_ioint* key,
			  char** val)
{
  osiv_ioint blen;
  return (OsivRead(&blen,sizeof(osiv_ioint),1,file) ||
	  OsivRead(key,sizeof(osiv_ioint),1,file) ||
	  (blen -= sizeof(osiv_ioint)) < 1 ||
	  !(*val = (char*)malloc(blen*sizeof(char))) ||
	  OsivRead(*val,sizeof(char),blen,file) ||
	  *(*val+blen-1) != '\0');
}

osiv_bool OsivReadFrameNum (osiv_file* file,
			    osiv_ioint* frame)
{
  osiv_ioint blen;
  return (OsivRead(&blen,sizeof(osiv_ioint),1,file) ||
	  OsivRead(frame,sizeof(osiv_ioint),1,file) ||
	  blen != sizeof(osiv_ioint));
}

osiv_bool OsivReadVectors (osiv_file* file,
			   osiv_ioint* len,
			   osiv_ioreal** val)
{
  osiv_ioint blen;
  return (OsivRead(&blen,sizeof(osiv_ioint),1,file) ||
	  OsivRead(len,sizeof(osiv_ioint),1,file) ||
	  !(*val = (osiv_ioreal*)malloc((*len)*sizeof(osiv_ioreal))) ||
	  OsivRead(*val,sizeof(osiv_ioreal),*len,file) ||
	  blen != sizeof(osiv_ioint)+(*len)*sizeof(osiv_ioreal));
}

osiv_bool OsivReadBoxNum (osiv_file* file,
			  osiv_ioint* val)
{
  osiv_ioint blen;
  return (OsivRead(&blen,sizeof(osiv_ioint),1,file) ||
	  OsivRead(val,sizeof(osiv_ioint),1,file) ||
	  blen != sizeof(osiv_ioint));
}

osiv_bool OsivReadMap (osiv_file* file,
		       osiv_ioint* len,
		       osiv_ioreal** map)
{
  osiv_ioint blen;
  return (OsivRead(&blen,sizeof(osiv_ioint),1,file) ||
	  OsivRead(len,sizeof(osiv_ioint),1,file) ||
	  !(*map = (osiv_ioreal*)malloc((*len)*sizeof(osiv_ioreal))) ||
	  OsivRead(*map,sizeof(osiv_ioreal),*len,file) ||
	  blen != sizeof(osiv_ioint)+(*len)*sizeof(osiv_ioreal));
}

osiv_bool OsivTagCmp (osiv_iotag a,
		      osiv_iotag b)
{
  return (a ^ b);
}

osiv_bool OsivScanToTag (osiv_file* file,
			 osiv_iotag tag)
{
  osiv_iotag top;

  while (OSIV_TRUE)
    {
      if (OsivRead(&top,sizeof(osiv_iotag),1,file))
	return OSIV_TRUE;

      if (top == tag)
	return OSIV_FALSE;

      if (top & OSIV_TAG_ISBASE)
	{
	  osiv_ioint blen;
	  osiv_iotag bottom;
	  if (OsivRead(&blen,sizeof(osiv_ioint),1,file) ||
	      OsivSeek(file,blen,SEEK_CUR) ||
	      OsivRead(&bottom,sizeof(osiv_iotag),1,file) ||
	      OsivTagCmp(bottom,~top))
	    return OSIV_TRUE;
	}
      else if (OsivScanToTag(file,~top))
	return OSIV_TRUE;
    }

  return OSIV_TRUE;
}

osiv_bool OsivScanToFrame (osiv_file* file,
			   osiv_intm num)
{
  long pos;
  osiv_ioint current;
  osiv_iotag closer;

  if (OsivScanToTag(file,OSIV_TAG_FRAME) ||
      OsivTell(file,&pos) ||
      OsivScanToTag(file,OSIV_TAG_FRAME_NUM) ||
      OsivReadFrameNum(file,&current) ||
      OsivReadTag(file,&closer) ||
      OsivTagCmp(closer,OSIV_GAT_FRAME_NUM))
    return OSIV_TRUE;

  while (current != num)
    {
      if (OsivScanToTag(file,OSIV_GAT_FRAME) ||
	  OsivScanToTag(file,OSIV_TAG_FRAME) ||
	  OsivTell(file,&pos) ||
	  OsivScanToTag(file,OSIV_TAG_FRAME_NUM) ||
	  OsivReadFrameNum(file,&current) ||
	  OsivReadTag(file,&closer) ||
	  OsivTagCmp(closer,OSIV_GAT_FRAME_NUM))
	return OSIV_TRUE;
    } 

  return OsivSeek(file,pos,SEEK_SET);
}

osiv_bool OsivScanToBox (osiv_file* file,
			 osiv_intf num)
{
  long pos;
  osiv_ioint current;
  osiv_iotag closer;

  if (OsivScanToTag(file,OSIV_TAG_BOX) ||
      OsivTell(file,&pos) ||
      OsivScanToTag(file,OSIV_TAG_BOX_NUM) ||
      OsivReadBoxNum(file,&current) ||
      OsivReadTag(file,&closer) ||
      OsivTagCmp(closer,OSIV_GAT_BOX_NUM))
    return OSIV_TRUE;

  while (current != num) 
    {
      if (OsivScanToTag(file,OSIV_GAT_BOX) ||
	  OsivScanToTag(file,OSIV_TAG_BOX) ||
	  OsivTell(file,&pos) ||
	  OsivScanToTag(file,OSIV_TAG_BOX_NUM) ||
	  OsivReadBoxNum(file,&current) ||
	  OsivReadTag(file,&closer) ||
	  OsivTagCmp(closer,OSIV_GAT_BOX_NUM))
	return OSIV_TRUE;
    }

  return OsivSeek(file,pos,SEEK_SET);
}

osiv_bool OsivGrabArgInt(osiv_file* file,
			 osiv_ioint key,
			 osiv_ioint* val)
{
  osiv_ioint current;
  osiv_iotag closer;

  do
    {
      if(OsivScanToTag(file,OSIV_TAG_ARG_INT) ||
	 OsivReadArgInt(file,&current,val) ||
	 OsivReadTag(file,&closer) ||
	 OsivTagCmp(closer,OSIV_GAT_ARG_INT))
	return OSIV_TRUE;
    } while (current != key);

  return OSIV_FALSE;
}

osiv_bool OsivGrabArgStr(osiv_file* file,
			 osiv_ioint key,
			 char** val)
{
  osiv_ioint current;
  osiv_iotag closer;

  do
    {
      if (OsivScanToTag(file,OSIV_TAG_ARG_STR) ||
	  OsivReadArgStr(file,&current,val) ||
	  OsivReadTag(file,&closer) ||
	  OsivTagCmp(closer,OSIV_GAT_ARG_STR))
	return OSIV_TRUE;
    } while (current != key);

  return OSIV_FALSE;
}

osiv_bool OsivGrabVectors (osiv_file* file,
			   osiv_intm frame,
			   osiv_ioint* vectors_len,
			   osiv_ioreal** vectors)
{
  osiv_iotag closer;
  return (OsivScanToFrame(file,frame) ||
	  OsivScanToTag(file,OSIV_TAG_VECTORS) ||
	  OsivReadVectors(file,vectors_len,vectors) ||
	  OsivReadTag(file,&closer) ||
	  OsivTagCmp(closer,OSIV_GAT_VECTORS));
}

osiv_bool OsivGrabMap (osiv_file* file,
		       osiv_intm frame,
		       osiv_intf window,
		       osiv_ioint* map_len,
		       osiv_ioreal** map)
{
  osiv_iotag closer;
  return (OsivScanToFrame(file,frame) ||
	  OsivScanToBox(file,window) ||
	  OsivScanToTag(file,OSIV_TAG_MAP) ||
	  OsivReadMap(file,map_len,map) ||
	  OsivReadTag(file,&closer) ||
	  OsivTagCmp(closer,OSIV_GAT_MAP));
}

