/* File: movie.c

* Copyright (c) 2002 James A. Strother
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/

#include "osiv.h"

static osiv_inta OsivSetupMovie_GetType(const char* str)
{
  char* suf;

  if (!strcmp(str,"-"))
    return OSIV_MOVIE_STREAM;

  if (strrchr(str,'%') != strchr(str,'%'))
    OsivError2("movie filename has multiple escapes: ",str);

  suf = strrchr(str,'.');
  if (!suf) OsivError2("movie filename needs extension: ",str);
  suf++;

  if (!strcmp(suf,"dat"))
    return OSIV_MOVIE_FILE;
  
  if (!strcmp(suf,"tif") || !strcmp(suf,"tiff"))
    {
      if (strchr(str,'%'))
	return OSIV_MOVIE_TIFF_SPLIT;
      else
	return OSIV_MOVIE_TIFF_MERGE;
    }

  OsivError2("can't parse movie filename extension: ",suf);
  return -1;
}

void OsivSetupMovie (osiv_param* p,
		     const char* fn)
{
  /* Grab type */
  p->MovieType = OsivSetupMovie_GetType(fn);

  /* Open necessary files */
  switch(p->MovieType)
    {
    case OSIV_MOVIE_STREAM:
      if (p->FrmSet != 1 || p->FrmSkip != 2)
	OsivError("movie from stdin requires set=1, skip=2");
      p->MovieData = (void*)stdin;
      break;
    case OSIV_MOVIE_FILE:
      p->MovieData = (void*)fopen(fn,"rb");
      break;
    case OSIV_MOVIE_TIFF_MERGE:
      p->MovieData = (void*)TIFFOpen(fn,"r");
      break;
    case OSIV_MOVIE_TIFF_SPLIT:
      p->MovieData = (void*)OsivMalloc(OSIV_STR_LEN+1);

      if (strlen(fn) > OSIV_STR_LEN)
	OsivError2("string too long: ",fn);

      strcpy((char*)p->MovieData,fn);
      break;
    default:
      OsivError("unrecognized movie file type");
      break;
    }

  if (!p->MovieData)
    OsivError("unable to open movie file");
}

static void OsivGetImage_GetTiff (TIFF* tif, 
				  osiv_intf xsize,
				  osiv_intf ysize,
				  osiv_imcp* image)
{
  tsize_t i;
  uint16_t nbits, nsamples;
  uint32_t length, width;

  TIFFGetField(tif,TIFFTAG_BITSPERSAMPLE,&nbits);
  if (nbits != CHAR_BIT*sizeof(osiv_imcp))
    OsivError("image depth doesn't match compile-time value");
 
  TIFFGetField(tif,TIFFTAG_SAMPLESPERPIXEL,&nsamples);
  if (nsamples != 1)
    OsivError("image doesn't appear to be grayscale");

  TIFFGetField(tif,TIFFTAG_IMAGEWIDTH,&width);
  if (width != xsize)
    OsivError("width of tiff image doesn't match parameter");

  TIFFGetField(tif,TIFFTAG_IMAGELENGTH,&length);
  if (length != ysize)
    OsivError("height of tiff image doesn't match parameter");

  if (TIFFIsTiled(tif)) 
    OsivError("tiled tiff images are unsupported");

  for (i = 0; i < ysize; i++)
    {
      if (TIFFReadScanline(tif,image,i,0) < 0)
	OsivError("unable to read tiff image");
      image+=xsize;
    }
}

void OsivGetImage (osiv_param* p,
		   osiv_intm frm,
		   osiv_imcp* im)
{
  switch(p->MovieType)
    {
    case OSIV_MOVIE_STREAM:
      {
	FILE* fp = p->MovieData;
	size_t len = p->ImageLength;

	if (fread(im,sizeof(osiv_imcp),len,fp) != len)
	  OsivError("unable to read image file");
      }
    case OSIV_MOVIE_FILE:
      {
	FILE* fp = p->MovieData;
	size_t len = p->ImageLength;
	
	if (fseek(fp,frm*len*sizeof(osiv_imcp),SEEK_SET) ||
	    fread(im,sizeof(osiv_imcp),len,fp) != len)
	  OsivError("unable to read image file");
      }
      break;
    case OSIV_MOVIE_TIFF_MERGE:
      {
	TIFF* fp = p->MovieData;
	
	TIFFSetDirectory(fp,frm);
	OsivGetImage_GetTiff(fp,p->XSize,p->YSize,im);
      }
      break;
    case OSIV_MOVIE_TIFF_SPLIT:
      {
	TIFF* fp;
	char* str;

	str = (char*)OsivMalloc(OSIV_STR_LEN+1);
	sprintf(str,(char*)p->MovieData,frm);

	fp = TIFFOpen(str,"r");
	if (!fp) OsivError("unable to open tiff file");

	OsivGetImage_GetTiff(fp,p->XSize,p->YSize,im);
	TIFFClose(fp);
	free(str);
      }
      break;
    default:
      OsivError("unable to find movie type");
      break;
    }
}

/* Destruction function, what's your conjunction? */
void OsivCloseMovie (osiv_param *p)
{
  switch(p->MovieType)
    {
    case OSIV_MOVIE_STREAM:
      if (fflush((FILE*)p->MovieData) == EOF)
	OsivError("unable to flush stream");
      break;
    case OSIV_MOVIE_FILE:
      if (fclose((FILE*)p->MovieData) == EOF)
	OsivError("unable to close data file");
      break;      
    case OSIV_MOVIE_TIFF_MERGE:
      TIFFClose((TIFF*)p->MovieData);
      break;
    case OSIV_MOVIE_TIFF_SPLIT:
      free(p->MovieData);
      break;
    default: 
      OsivError("unrecognized movie format");
      break;
    }
}
