/* File: parse.c 
 
 * Copyright (c) 2002 James A. Strother 
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License  
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 * 
*/ 
 
#include "osiv.h" 
 
static void OsivParseInput_ParseOpts (osiv_input*,int,char**); 
 
static void OsivParseInput_BreakArgs (int argc, 
				      char** args) 
{ 
  int i; 
 
  for (i = 0; i < argc; i++) 
    { 
      char* c = strchr(*(args+i),'='); 
      if (!c) 
	OsivError2("parameter has no argument: ",*(args+i)); 
      else 
	*c = '\0'; 
    } 
} 
 
static osiv_bool OsivParseInput_IsBlank (const char* a) 
{ 
  return (strlen(a) == strspn(a," \n\t")); 
} 
 
static osiv_bool OsivParseInput_PullNum (const char* arg, 
					 const char* param, 
					 osiv_ioint* out) 
{ 
  const char* key = arg; 
  const char* val = arg+strlen(arg)+1; 
 
  if (strcmp(key,param)) return OSIV_FALSE; 
  else 
    { 
      char* endptr; 
     
      *out = (osiv_ioint)strtol(val,&endptr,10); 
      if (*endptr != '\0') 
	OsivError2("unable to interpret number: ",val); 
    } 
  return OSIV_TRUE; 
} 
 
static osiv_bool OsivParseInput_PullStr (const char* arg, 
					 const char* param, 
					 char* out) 
{ 
  const char* key = arg; 
  const char* val = arg+strlen(arg)+1; 
 
  if (strcmp(key,param)) 
    return OSIV_FALSE; 
   
  if (strlen(val) > OSIV_STR_LEN) 
    OsivError2("string is too long: ",val); 
				 
  strcpy(out,val); 
  return OSIV_TRUE; 
} 
 
static osiv_bool OsivParseInput_PullParam (const char* arg, 
					   const char* param, 
					   osiv_input* out) 
{ 
  const char* key = arg; 
  const char* val = arg+strlen(arg)+1; 
 
  if (strcmp(key,param)) return OSIV_FALSE; 
  else 
    { 
      FILE* fptr; 
      char* fmt = (char*)OsivMalloc(OSIV_STR_LEN+1); 
      char* prm = (char*)OsivMalloc(OSIV_STR_LEN+1); 
 
      strcpy(fmt,"%"); 
      sprintf(fmt+1,"%d",OSIV_STR_LEN); 
      strcat(fmt,"s"); 
 
      if (!(fptr = fopen(val,"r"))) 
	OsivError("unable to open parameter file"); 
			 
      while (!feof(fptr))	 
	{ 
	  if (fscanf(fptr,fmt,prm) != EOF) 
	    OsivParseInput_ParseOpts(out,1,&prm); 
	} 
			 
      if (fclose(fptr) == EOF) 
	OsivError("unable to close parameter file"); 
 
      free(fmt); 
      free(prm); 
    } 
 
  return OSIV_TRUE; 
} 
 
static osiv_bool OsivParseInput_PullWisdom (const char* arg, 
					    const char* param) 
{ 
  const char* key = arg; 
  const char* val = arg+strlen(arg)+1; 
 
  if (strcmp(key,param)) return OSIV_FALSE; 
  else 
    { 
      FILE* fptr = fopen(val,"r"); 
      if (!fptr) 
	OsivError("unable to open wisdom file"); 
 
      if (!fftw_import_wisdom_from_file(fptr)) 
	OsivError("unable to read wisdom file"); 
 
      if (fclose(fptr) == EOF) 
	OsivError("unable to close wisdom file"); 
    } 
 
  return OSIV_TRUE; 
} 
 
static void OsivParseInput_ParseOpts (osiv_input* input, 
				      int argc, 
				      char** args) 
{ 
  int i; 
 
  /* Distinguish keys and values */ 
  OsivParseInput_BreakArgs(argc,args); 
 
  /* Sort out the parameters */ 
  for (i = 0; i < argc; i++) 
    { 
      if (!( 
	    OsivParseInput_PullNum(*(args+i),"alg",&(input->Alg)) || 
	    OsivParseInput_PullNum(*(args+i),"ppr",&(input->PreProc)) || 
	    OsivParseInput_PullNum(*(args+i),"itp",&(input->Interp)) || 
	    OsivParseInput_PullNum(*(args+i),"szx",&(input->XSize)) || 
	    OsivParseInput_PullNum(*(args+i),"szy",&(input->YSize)) || 
	    OsivParseInput_PullNum(*(args+i),"stx",&(input->XStart)) || 
	    OsivParseInput_PullNum(*(args+i),"sty",&(input->YStart)) || 
	    OsivParseInput_PullNum(*(args+i),"ofx",&(input->XOffset)) || 
	    OsivParseInput_PullNum(*(args+i),"ofy",&(input->YOffset)) || 
	    OsivParseInput_PullNum(*(args+i),"spx",&(input->XVectSpace)) || 
	    OsivParseInput_PullNum(*(args+i),"spy",&(input->YVectSpace)) || 
	    OsivParseInput_PullNum(*(args+i),"nmx",&(input->XNumVects)) || 
	    OsivParseInput_PullNum(*(args+i),"nmy",&(input->YNumVects)) || 
	    OsivParseInput_PullNum(*(args+i),"wsx",&(input->XWinSize)) || 
	    OsivParseInput_PullNum(*(args+i),"wsy",&(input->YWinSize)) || 
	    OsivParseInput_PullNum(*(args+i),"mdx",&(input->XMaxDisp)) || 
	    OsivParseInput_PullNum(*(args+i),"mdy",&(input->YMaxDisp)) || 
	    OsivParseInput_PullNum(*(args+i),"wlx",&(input->XWinLgSize)) || 
	    OsivParseInput_PullNum(*(args+i),"wly",&(input->YWinLgSize)) || 
	    OsivParseInput_PullNum(*(args+i),"sfr",&(input->FrmStart)) || 
	    OsivParseInput_PullNum(*(args+i),"efr",&(input->FrmFinish)) || 
	    OsivParseInput_PullNum(*(args+i),"skp",&(input->FrmSkip)) || 
	    OsivParseInput_PullNum(*(args+i),"set",&(input->FrmSet)) || 
	    OsivParseInput_PullNum(*(args+i),"nit",&(input->NumIter)) || 
	    OsivParseInput_PullNum(*(args+i),"nth",&(input->NumThreads)) || 
	    OsivParseInput_PullNum(*(args+i),"wmp",&(input->WriteMap)) || 
	    OsivParseInput_PullStr(*(args+i),"mvn",input->MovieFN) || 
	    OsivParseInput_PullStr(*(args+i),"dtn",input->VectorsFN) || 
	    OsivParseInput_PullParam(*(args+i),"prn",input) || 
	    OsivParseInput_PullWisdom(*(args+i),"wsd") || 
	    OsivParseInput_IsBlank(*(args+i)) 
	    )) 
	OsivError2("unrecognized parameter type: ",*(args+i)); 
    } 
} 
 
static void OsivParseInput_CheckOpts (osiv_input* input) 
{ 
  if (input->Alg == OSIV_IOINT_BLANK) 
    OsivError("alg parameter is missing"); 
 
  switch(input->Alg) 
    { 
    case 1: OsivAlg1_CheckOpts(input); break; 
    case 2: OsivAlg2_CheckOpts(input); break; 
    case 3: OsivAlg3_CheckOpts(input); break; 
    case 4: OsivAlg4_CheckOpts(input); break; 
    case 5: OsivAlg5_CheckOpts(input); break; 
    case 6: OsivAlg6_CheckOpts(input); break; 
    case 7: OsivAlg7_CheckOpts(input); break; 
    case 8: OsivAlg8_CheckOpts(input); break; 
#ifndef OSIV_NO_THREADS 
    case 9: OsivAlg9_CheckOpts(input);break; 
#endif 
    default: OsivError("unrecognized algorithm number"); break; 
    } 
} 
 
void OsivParseInput (osiv_input* input, 
		     int argc, 
		     char** args) 
{ 
  /* Set null values */ 
  input->Alg = OSIV_IOINT_BLANK; 
  input->PreProc = OSIV_IOINT_BLANK; 
  input->Interp = OSIV_IOINT_BLANK; 
  input->XSize = OSIV_IOINT_BLANK; 
  input->YSize = OSIV_IOINT_BLANK; 
  input->XStart = OSIV_IOINT_BLANK; 
  input->YStart = OSIV_IOINT_BLANK; 
  input->XOffset = OSIV_IOINT_BLANK; 
  input->YOffset = OSIV_IOINT_BLANK; 
  input->XVectSpace = OSIV_IOINT_BLANK; 
  input->YVectSpace = OSIV_IOINT_BLANK; 
  input->XNumVects = OSIV_IOINT_BLANK; 
  input->YNumVects = OSIV_IOINT_BLANK; 
  input->XWinSize = OSIV_IOINT_BLANK; 
  input->YWinSize = OSIV_IOINT_BLANK; 
  input->XMaxDisp = OSIV_IOINT_BLANK; 
  input->YMaxDisp = OSIV_IOINT_BLANK; 
  input->XWinLgSize = OSIV_IOINT_BLANK; 
  input->YWinLgSize = OSIV_IOINT_BLANK; 
  input->FrmStart = OSIV_IOINT_BLANK; 
  input->FrmFinish = OSIV_IOINT_BLANK; 
  input->FrmSkip = OSIV_IOINT_BLANK; 
  input->FrmSet = OSIV_IOINT_BLANK; 
  input->NumIter = OSIV_IOINT_BLANK; 
  input->NumThreads = OSIV_IOINT_BLANK; 
  input->WriteMap = OSIV_IOINT_BLANK; 
 
  input->MovieFN = (char*)OsivMalloc(OSIV_STR_LEN+1); 
  input->VectorsFN = (char*)OsivMalloc(OSIV_STR_LEN+1); 
 
  memset(input->MovieFN,'\0',OSIV_STR_LEN+1); 
  memset(input->VectorsFN,'\0',OSIV_STR_LEN+1); 
 
  /* Interpret strings */ 
  OsivParseInput_ParseOpts(input,argc,args); 
   
  /* Check for untenable parameters */ 
  OsivParseInput_CheckOpts(input); 
} 
